@@ -1,4 +1,16 @@
 package gl4gcj;
+/** \mainpage
+* gl4gcj is a GCJ(http://gcc.gnu.org/java/) Java binding to OpenGL.
+* 
+* Packages:
+* <dl>
+*   <dt> gl4gcj </dt>
+*   <dd> OpenGL gcj binding.</dd>
+*   <dt> glut4gcj</dt>
+*   <dd> GLUT gcj binding.</dd>
+* </dl>
+**/
+
 public class GL implements GLConstants
 {
 	// Miscellaneous
@@ -375,14 +387,14 @@ public class GL implements GLConstants
 	//?? dangerous...  no pixels size check
 	public static native void glReadPixels(int x, int y, int width, int height, int format, int type, float[] pixels);
 	public static native void glReadPixels(int x, int y, int width, int height, int format, int type, byte[] pixels);
-	public static native void glReadPixels(int x, int y, int width, int height, int format, int type, int[] pixels);
 	public static native void glReadPixels(int x, int y, int width, int height, int format, int type, short[] pixels);
+	public static native void glReadPixels(int x, int y, int width, int height, int format, int type, int[] pixels);
 
 	//?? warning...  no values size check
 	public static native void glDrawPixels(int width, int height, int format, int type, float[] pixels);
 	public static native void glDrawPixels(int width, int height, int format, int type, byte[] pixels);
-	public static native void glDrawPixels(int width, int height, int format, int type, int[] pixels);
 	public static native void glDrawPixels(int width, int height, int format, int type, short[] pixels);
+	public static native void glDrawPixels(int width, int height, int format, int type, int[] pixels);
 
 	public static native void glCopyPixels(int x, int y, int width, int height, int bufferType);
 
@@ -407,30 +419,68 @@ public class GL implements GLConstants
 	public static native void glGetTexGenfv(int coord, int pname, float[] params);
 	public static native void glGetTexGeniv(int coord, int pname, int[] params);
 
-/*
 	public static native void glTexEnvf(int target, int pname, float param);
 	public static native void glTexEnvi(int target, int pname, int param);
+
+	//?? warning...  no checking size of params.
 	public static native void glTexEnvfv(int target, int pname, float[] params);
 	public static native void glTexEnviv(int target, int pname, int[] params);
+
+	//?? dangerous...  no checking size of params.
 	public static native void glGetTexEnvfv(int target, int pname, float[] params);
 	public static native void glGetTexEnviv(int target, int pname, int[] params);
+
 	public static native void glTexParameterf(int target, int pname, float param);
 	public static native void glTexParameteri(int target, int pname, int param);
+
+	//?? warning...  no checking size of params.
 	public static native void glTexParameterfv(int target, int pname, float[] params);
 	public static native void glTexParameteriv(int target, int pname, int[] params);
+
+	//?? dangerous...  no checking size of params.
 	public static native void glGetTexParameterfv(int target, int pname, float[] params);
 	public static native void glGetTexParameteriv(int target, int pname, int[] params);
 	public static native void glGetTexLevelParameterfv(int target, int level, int pname, float[] params);
 	public static native void glGetTexLevelParameteriv(int target, int level, int pname, int[] params);
-	public static native void glTexImage1D(int target, int level, int internalFormat, int width, int border, int format, int type, Object[] pixels);
-	public static native void glTexImage2D(int target, int level, int internalFormat, int width, int height, int border, int format, int type, Object[] pixels);
-	public static native void glGetTexImage(int target, int level, int format, int type, Object[] pixels);
+
+	//?? warning...  no checking size of pixels.
+	public static native void glTexImage1D(int target, int level, int internalFormat, int width, int border, int format, int type, float[] pixels);
+	public static native void glTexImage1D(int target, int level, int internalFormat, int width, int border, int format, int type, byte[] pixels);
+	public static native void glTexImage1D(int target, int level, int internalFormat, int width, int border, int format, int type, short[] pixels);
+	public static native void glTexImage1D(int target, int level, int internalFormat, int width, int border, int format, int type, int[] pixels);
+
+	public static native void glTexImage2D(int target, int level, int internalFormat, int width, int height, int border, int format, int type, float[] pixels);
+	public static native void glTexImage2D(int target, int level, int internalFormat, int width, int height, int border, int format, int type, byte[] pixels);
+	public static native void glTexImage2D(int target, int level, int internalFormat, int width, int height, int border, int format, int type, short[] pixels);
+	public static native void glTexImage2D(int target, int level, int internalFormat, int width, int height, int border, int format, int type, int[] pixels);
+
+	//?? dangerous...  no checking size of pixels.
+	public static native void glGetTexImage(int target, int level, int format, int type, float[] pixels);
+	public static native void glGetTexImage(int target, int level, int format, int type, byte[] pixels);
+	public static native void glGetTexImage(int target, int level, int format, int type, short[] pixels);
+	public static native void glGetTexImage(int target, int level, int format, int type, int[] pixels);
 
 	// 1.1 functions 
+	public static int[] glGenTextures(int num)
+	{
+		int[] texstureNameArray = new int[num];
+		glGenTextures(num, texstureNameArray);
+		return texstureNameArray;
+	}
+	public static void glGenTextures(int[] texstureNameArray)
+	{
+		glGenTextures(texstureNameArray.length, texstureNameArray);
+	}
+	public static native void glGenTextures(int n, int[] texstureNameArray);
 
-	public static native void glGenTextures(int n, long[] textures);
-	public static native void glDeleteTextures(int n, long[] textures);
-	public static native void glBindTexture(int target, long texture);
+	public static void glDeleteTextures(int[] texstureNameArray)
+	{
+		glDeleteTextures(texstureNameArray.length, texstureNameArray);
+	}
+	public static native void glDeleteTextures(int n, int[] texstureNameArray);
+	public static native void glBindTexture(int target, int texture);
+
+/*
 	public static native void glPrioritizeTextures(int n, long[] textures, float[] priorities);
 	public static native byte glAreTexturesResident(int n, long[] textures, byte[] residences);
 	public static native byte glIsTexture(long texture);
