@@ -2104,7 +2104,7 @@ GL::glDrawElements (jint mode, jbyteArray indices)
 {
 	JvInitClass(&GL::class$);
 	::glDrawElements (mode, JvGetArrayLength(indices), 
-		GL_BYTE, elements(indices));
+		GL_UNSIGNED_BYTE, elements(indices));
 }
 
 void
@@ -2112,7 +2112,7 @@ GL::glDrawElements (jint mode, jshortArray indices)
 {
 	JvInitClass(&GL::class$);
 	::glDrawElements (mode, JvGetArrayLength(indices), 
-		GL_SHORT, elements(indices));
+		GL_UNSIGNED_SHORT, elements(indices));
 }
 void
 
@@ -2120,7 +2120,7 @@ GL::glDrawElements (jint mode, jintArray indices)
 {
 	JvInitClass(&GL::class$);
 	::glDrawElements (mode, JvGetArrayLength(indices), 
-		GL_INT, elements(indices));
+		GL_UNSIGNED_INT, elements(indices));
 }
 
 /*
@@ -2531,148 +2531,231 @@ GL::glGetTexGeniv (jint coord, jint pname, jintArray params)
 	::glGetTexGeniv(coord, pname, (GLint*)elements(params));
 }
 
-/*
 
 void
-GL::glTexEnvf (jint, jint, jfloat)
+GL::glTexEnvf (jint target, jint pname, jfloat param)
 {
 	JvInitClass(&GL::class$);
+	::glTexEnvf(target, pname, param);
 }
 
 
 void
-GL::glTexEnvi (jint, jint, jint)
+GL::glTexEnvi (jint target, jint pname, jint param)
 {
 	JvInitClass(&GL::class$);
+	::glTexEnvi(target, pname, param);
 }
 
 
 void
-GL::glTexEnvfv (jint, jint, jfloatArray)
+GL::glTexEnvfv (jint target, jint pname, jfloatArray params)
 {
 	JvInitClass(&GL::class$);
+	::glTexEnvfv(target, pname, (GLfloat*)elements(params));
 }
 
 
 void
-GL::glTexEnviv (jint, jint, jintArray)
+GL::glTexEnviv (jint target, jint pname, jintArray params)
 {
 	JvInitClass(&GL::class$);
+	::glTexEnviv(target, pname, (GLint*)elements(params));
 }
 
 
 void
-GL::glGetTexEnvfv (jint, jint, jfloatArray)
+GL::glGetTexEnvfv (jint target, jint pname, jfloatArray params)
 {
 	JvInitClass(&GL::class$);
+	::glGetTexEnvfv(target, pname, (GLfloat*)elements(params));
 }
 
 
 void
-GL::glGetTexEnviv (jint, jint, jintArray)
+GL::glGetTexEnviv (jint target, jint pname, jintArray params)
 {
 	JvInitClass(&GL::class$);
+	::glGetTexEnviv(target, pname, (GLint*)elements(params));
+}
+
+void
+GL::glTexParameterf (jint target, jint pname, jfloat param)
+{
+	JvInitClass(&GL::class$);
+	::glTexParameterf(target, pname, param);
 }
 
 
 void
-GL::glTexParameterf (jint, jint, jfloat)
+GL::glTexParameteri (jint target, jint pname, jint param)
 {
 	JvInitClass(&GL::class$);
+	::glTexParameteri(target, pname, param);
 }
 
 
 void
-GL::glTexParameteri (jint, jint, jint)
+GL::glTexParameterfv (jint target, jint pname, jfloatArray params)
 {
 	JvInitClass(&GL::class$);
+	::glTexParameterfv(target, pname, (GLfloat*)elements(params));
 }
 
 
 void
-GL::glTexParameterfv (jint, jint, jfloatArray)
+GL::glTexParameteriv (jint target, jint pname, jintArray params)
 {
 	JvInitClass(&GL::class$);
+	::glTexParameteriv(target, pname, (GLint*)elements(params));
 }
 
 
 void
-GL::glTexParameteriv (jint, jint, jintArray)
+GL::glGetTexParameterfv (jint target, jint pname, jfloatArray params)
 {
 	JvInitClass(&GL::class$);
+	::glGetTexParameterfv(target, pname, (GLfloat*)elements(params));
+}
+
+void
+GL::glGetTexParameteriv (jint target, jint pname, jintArray params)
+{
+	JvInitClass(&GL::class$);
+	::glGetTexParameteriv(target, pname, (GLint*)elements(params));
+}
+
+void
+GL::glGetTexLevelParameterfv (jint target, jint level, jint pname, jfloatArray params)
+{
+	JvInitClass(&GL::class$);
+	::glGetTexLevelParameterfv(target, level, pname, (GLfloat*)elements(params));
 }
 
 
 void
-GL::glGetTexParameterfv (jint, jint, jfloatArray)
+GL::glGetTexLevelParameteriv (jint target, jint level, jint pname, jintArray params)
 {
 	JvInitClass(&GL::class$);
+	::glGetTexLevelParameteriv(target, level, pname, (GLint*)elements(params));
+}
+
+void
+GL::glTexImage1D (jint target, jint level, jint internalFormat, jint width, jint border, jint format, jint type, jfloatArray pixels)
+{
+	JvInitClass(&GL::class$);
+	::glTexImage1D(target, level, internalFormat, width, border, format, type, (const void*)elements(pixels));
+}
+
+void
+GL::glTexImage1D (jint target, jint level, jint internalFormat, jint width, jint border, jint format, jint type, jbyteArray pixels)
+{
+	JvInitClass(&GL::class$);
+	::glTexImage1D(target, level, internalFormat, width, border, format, type, (const void*)elements(pixels));
+}
+
+void
+GL::glTexImage1D (jint target, jint level, jint internalFormat, jint width, jint border, jint format, jint type, jshortArray pixels)
+{
+	JvInitClass(&GL::class$);
+	::glTexImage1D(target, level, internalFormat, width, border, format, type, (const void*)elements(pixels));
+}
+
+void
+GL::glTexImage1D (jint target, jint level, jint internalFormat, jint width, jint border, jint format, jint type, jintArray pixels)
+{
+	JvInitClass(&GL::class$);
+	::glTexImage1D(target, level, internalFormat, width, border, format, type, (const void*)elements(pixels));
 }
 
 
 void
-GL::glGetTexParameteriv (jint, jint, jintArray)
+GL::glTexImage2D (jint target, jint level, jint internalFormat, jint width, jint height, jint border, jint format, jint type, jfloatArray pixels)
 {
 	JvInitClass(&GL::class$);
+	::glTexImage2D(target, level, internalFormat, width, height, border, format, type, (const void*)elements(pixels));
+}
+
+void
+GL::glTexImage2D (jint target, jint level, jint internalFormat, jint width, jint height, jint border, jint format, jint type, jbyteArray pixels)
+{
+	JvInitClass(&GL::class$);
+	::glTexImage2D(target, level, internalFormat, width, height, border, format, type, (const void*)elements(pixels));
+}
+
+void
+GL::glTexImage2D (jint target, jint level, jint internalFormat, jint width, jint height, jint border, jint format, jint type, jshortArray pixels)
+{
+	JvInitClass(&GL::class$);
+	::glTexImage2D(target, level, internalFormat, width, height, border, format, type, (const void*)elements(pixels));
+}
+
+void
+GL::glTexImage2D (jint target, jint level, jint internalFormat, jint width, jint height, jint border, jint format, jint type, jintArray pixels)
+{
+	JvInitClass(&GL::class$);
+	::glTexImage2D(target, level, internalFormat, width, height, border, format, type, (const void*)elements(pixels));
 }
 
 
 void
-GL::glGetTexLevelParameterfv (jint, jint, jint, jfloatArray)
+GL::glGetTexImage (jint target, jint level, jint format, jint type, jfloatArray pixels)
 {
 	JvInitClass(&GL::class$);
+	::glGetTexImage(target, level, format, type, elements(pixels));
+}
+
+void
+GL::glGetTexImage (jint target, jint level, jint format, jint type, jbyteArray pixels)
+{
+	JvInitClass(&GL::class$);
+	::glGetTexImage(target, level, format, type, elements(pixels));
+}
+
+void
+GL::glGetTexImage (jint target, jint level, jint format, jint type, jshortArray pixels)
+{
+	JvInitClass(&GL::class$);
+	::glGetTexImage(target, level, format, type, elements(pixels));
+}
+
+void
+GL::glGetTexImage (jint target, jint level, jint format, jint type, jintArray pixels)
+{
+	JvInitClass(&GL::class$);
+	::glGetTexImage(target, level, format, type, elements(pixels));
+}
+
+
+// 1.1 functions 
+void
+GL::glGenTextures (jint n, jintArray textureNameArray)
+{
+	JvInitClass(&GL::class$);
+	if (JvGetArrayLength(textureNameArray) < n) throw createArraySizeException();
+	GLuint* textures = (GLuint*)elements(textureNameArray);
+	::glGenTextures(n, textures);
 }
 
 
 void
-GL::glGetTexLevelParameteriv (jint, jint, jint, jintArray)
+GL::glDeleteTextures (jint n, jintArray textureNameArray)
 {
 	JvInitClass(&GL::class$);
+	if (JvGetArrayLength(textureNameArray) < n) throw createArraySizeException();
+	GLuint* textures = (GLuint*)elements(textureNameArray);
+	::glDeleteTextures(n, textures);
 }
 
-
 void
-GL::glTexImage1D (jint, jint, jint, jint, jint, jint, jint, JArray< ::java::lang::Object *> *)
-{
-	JvInitClass(&GL::class$);
-}
-
-
-void
-GL::glTexImage2D (jint, jint, jint, jint, jint, jint, jint, jint, JArray< ::java::lang::Object *> *)
-{
-	JvInitClass(&GL::class$);
-}
-
-
-void
-GL::glGetTexImage (jint, jint, jint, jint, JArray< ::java::lang::Object *> *)
-{
-	JvInitClass(&GL::class$);
-}
-
-
-void
-GL::glGenTextures (jint, jlongArray)
-{
-	JvInitClass(&GL::class$);
-}
-
-
-void
-GL::glDeleteTextures (jint, jlongArray)
-{
-	JvInitClass(&GL::class$);
-}
-
-
-void
-GL::glBindTexture (jint, jlong)
+GL::glBindTexture (jint target, jint texture)
 {
 	JvInitClass(&GL::class$);
+	::glBindTexture(target, texture);
 }
 
 
+/************************************************************
 void
 GL::glPrioritizeTextures (jint, jlongArray, jfloatArray)
 {
