@@ -18,6 +18,22 @@
 #include "glut4gcj/SpaceballRotateFunc.h"
 #include "glut4gcj/SpaceballButtonFunc.h"
 
+#include "glut4gcj/ButtonBoxFunc.h"
+#include "glut4gcj/DialsFunc.h"
+#include "glut4gcj/TabletMotionFunc.h"
+#include "glut4gcj/TabletButtonFunc.h"
+
+#include "glut4gcj/MenuStatusFunc.h"
+#include "glut4gcj/OverlayDisplayFunc.h"
+#include "glut4gcj/WindowStatusFunc.h"
+
+#include "glut4gcj/KeyboardUpFunc.h"
+#include "glut4gcj/SpecialUpFunc.h"
+#include "glut4gcj/JoystickFunc.h"
+
+#include "glut4gcj/BitmapFont.h"
+#include "glut4gcj/StrokeFont.h"
+
 #include <gcj/cni.h>
 #include <java/lang/Object.h>
 #include <java/lang/Integer.h>
@@ -27,6 +43,7 @@
 #include <stdlib.h>
 #include <stdio.h>
 
+
 using namespace ::glut4gcj;
 using namespace ::java::lang;
 using namespace ::java::util;
@@ -58,7 +75,7 @@ GLUT::glutInit (JArray< ::java::lang::String *> *argArray)
 			abort();
 		}
 		memcpy(buffer, bytes, byteArray->length);
-		bytes[byteArray->length] = '\0';
+		buffer[byteArray->length] = '\0';
 		argv[i] = buffer;
 		argv_backup[i] = buffer;
 	}
@@ -635,7 +652,7 @@ GLUT::glutKeyboardFunc (::glut4gcj::KeyboardFunc *func)
 	}
 	else
 	{
-		reshapeFuncs[win] = NULL;
+		keyboardFuncs[win] = NULL;
 		::glutKeyboardFunc(NULL);
 	}
 }
@@ -1223,6 +1240,506 @@ GLUT::glutSpaceballButtonFunc (::glut4gcj::SpaceballButtonFunc *func)
 }
 
 
+#if (GLUT_API_VERSION >= 2)
+static ButtonBoxFunc** buttonBoxFuncs = NULL;
+static int buttonBoxFuncsLength = 0;
+
+static 
+void 
+buttonBox(int button, int state)
+{
+	int win = ::glutGetWindow();
+	ButtonBoxFunc* buttonBoxFunc = buttonBoxFuncs[win];
+	if (buttonBoxFunc != NULL) buttonBoxFunc->buttonBox(button, state);
+}
+#endif
+
+void 
+GLUT::glutButtonBoxFunc (::glut4gcj::ButtonBoxFunc *func)
+{
+	JvInitClass(&GLUT::class$);
+#if (GLUT_API_VERSION >= 2)
+	int win = ::glutGetWindow();
+	if (win >= buttonBoxFuncsLength)
+	{
+		int newLength = buttonBoxFuncsLength * 2;
+		if (win >= newLength) newLength = win * 2 + 2;
+		ButtonBoxFunc** newFuncs = (ButtonBoxFunc**)malloc(sizeof(ButtonBoxFunc*) * newLength);  
+		if(newFuncs == NULL)
+		{
+			fprintf(stderr, "%s: %d: malloc() fail.\n", __FILE__, __LINE__);
+			abort();
+		}
+		
+		memcpy(newFuncs, buttonBoxFuncs, sizeof(ButtonBoxFunc*) * buttonBoxFuncsLength);
+		free(buttonBoxFuncs);
+		buttonBoxFuncs = newFuncs;
+		buttonBoxFuncsLength = newLength;
+	}
+
+	if (func != NULL)
+	{
+		buttonBoxFuncs[win] = func;
+		::glutButtonBoxFunc(buttonBox);
+	}
+	else
+	{
+		buttonBoxFuncs[win] = NULL;
+		::glutButtonBoxFunc(NULL);
+	}
+#endif
+}
+
+
+#if (GLUT_API_VERSION >= 2)
+static DialsFunc** dialsFuncs = NULL;
+static int dialsFuncsLength = 0;
+
+static 
+void 
+dials(int button, int state)
+{
+	int win = ::glutGetWindow();
+	DialsFunc* dialsFunc = dialsFuncs[win];
+	if (dialsFunc != NULL) dialsFunc->dials(button, state);
+}
+#endif
+
+void 
+GLUT::glutDialsFunc (::glut4gcj::DialsFunc *func)
+{
+	JvInitClass(&GLUT::class$);
+#if (GLUT_API_VERSION >= 2)
+	int win = ::glutGetWindow();
+	if (win >= dialsFuncsLength)
+	{
+		int newLength = dialsFuncsLength * 2;
+		if (win >= newLength) newLength = win * 2 + 2;
+		DialsFunc** newFuncs = (DialsFunc**)malloc(sizeof(DialsFunc*) * newLength);  
+		if(newFuncs == NULL)
+		{
+			fprintf(stderr, "%s: %d: malloc() fail.\n", __FILE__, __LINE__);
+			abort();
+		}
+		
+		memcpy(newFuncs, dialsFuncs, sizeof(DialsFunc*) * dialsFuncsLength);
+		free(dialsFuncs);
+		dialsFuncs = newFuncs;
+		dialsFuncsLength = newLength;
+	}
+
+	if (func != NULL)
+	{
+		dialsFuncs[win] = func;
+		::glutDialsFunc(dials);
+	}
+	else
+	{
+		dialsFuncs[win] = NULL;
+		::glutDialsFunc(NULL);
+	}
+#endif
+}
+
+#if (GLUT_API_VERSION >= 2)
+static TabletMotionFunc** tabletMotionFuncs = NULL;
+static int tabletMotionFuncsLength = 0;
+
+static 
+void 
+tabletMotion(int x, int y)
+{
+	int win = ::glutGetWindow();
+	TabletMotionFunc* tabletMotionFunc = tabletMotionFuncs[win];
+	if (tabletMotionFunc != NULL) tabletMotionFunc->tabletMotion(x, y);
+}
+#endif
+
+void 
+GLUT::glutTabletMotionFunc (::glut4gcj::TabletMotionFunc *func)
+{
+	JvInitClass(&GLUT::class$);
+#if (GLUT_API_VERSION >= 2)
+	int win = ::glutGetWindow();
+	if (win >= tabletMotionFuncsLength)
+	{
+		int newLength = tabletMotionFuncsLength * 2;
+		if (win >= newLength) newLength = win * 2 + 2;
+		TabletMotionFunc** newFuncs = (TabletMotionFunc**)malloc(sizeof(TabletMotionFunc*) * newLength);  
+		if(newFuncs == NULL)
+		{
+			fprintf(stderr, "%s: %d: malloc() fail.\n", __FILE__, __LINE__);
+			abort();
+		}
+		
+		memcpy(newFuncs, tabletMotionFuncs, sizeof(TabletMotionFunc*) * tabletMotionFuncsLength);
+		free(tabletMotionFuncs);
+		tabletMotionFuncs = newFuncs;
+		tabletMotionFuncsLength = newLength;
+	}
+
+	if (func != NULL)
+	{
+		tabletMotionFuncs[win] = func;
+		::glutTabletMotionFunc(tabletMotion);
+	}
+	else
+	{
+		tabletMotionFuncs[win] = NULL;
+		::glutTabletMotionFunc(NULL);
+	}
+#endif
+}
+
+#if (GLUT_API_VERSION >= 2)
+static TabletButtonFunc** tabletButtonFuncs = NULL;
+static int tabletButtonFuncsLength = 0;
+
+static 
+void 
+tabletButton(int button, int state, int x, int y)
+{
+	int win = ::glutGetWindow();
+	TabletButtonFunc* tabletButtonFunc = tabletButtonFuncs[win];
+	if (tabletButtonFunc != NULL) tabletButtonFunc->tabletButton(button, state, x, y);
+}
+#endif
+
+void 
+GLUT::glutTabletButtonFunc (::glut4gcj::TabletButtonFunc *func)
+{
+	JvInitClass(&GLUT::class$);
+#if (GLUT_API_VERSION >= 2)
+	int win = ::glutGetWindow();
+	if (win >= tabletButtonFuncsLength)
+	{
+		int newLength = tabletButtonFuncsLength * 2;
+		if (win >= newLength) newLength = win * 2 + 2;
+		TabletButtonFunc** newFuncs = (TabletButtonFunc**)malloc(sizeof(TabletButtonFunc*) * newLength);  
+		if(newFuncs == NULL)
+		{
+			fprintf(stderr, "%s: %d: malloc() fail.\n", __FILE__, __LINE__);
+			abort();
+		}
+		
+		memcpy(newFuncs, tabletButtonFuncs, sizeof(TabletButtonFunc*) * tabletButtonFuncsLength);
+		free(tabletButtonFuncs);
+		tabletButtonFuncs = newFuncs;
+		tabletButtonFuncsLength = newLength;
+	}
+
+	if (func != NULL)
+	{
+		tabletButtonFuncs[win] = func;
+		::glutTabletButtonFunc(tabletButton);
+	}
+	else
+	{
+		tabletButtonFuncs[win] = NULL;
+		::glutTabletButtonFunc(NULL);
+	}
+#endif
+}
+
+
+#if (GLUT_API_VERSION >= 3)
+static MenuStatusFunc** menuStatusFuncs = NULL;
+static int menuStatusFuncsLength = 0;
+
+static 
+void 
+menuStatus(int status, int x, int y)
+{
+	int win = ::glutGetWindow();
+	MenuStatusFunc* menuStatusFunc = menuStatusFuncs[win];
+	if (menuStatusFunc != NULL) menuStatusFunc->menuStatus(status, x, y);
+}
+#endif
+
+void 
+GLUT::glutMenuStatusFunc (::glut4gcj::MenuStatusFunc *func)
+{
+	JvInitClass(&GLUT::class$);
+#if (GLUT_API_VERSION >= 3)
+	int win = ::glutGetWindow();
+	if (win >= menuStatusFuncsLength)
+	{
+		int newLength = menuStatusFuncsLength * 2;
+		if (win >= newLength) newLength = win * 2 + 2;
+		MenuStatusFunc** newFuncs = (MenuStatusFunc**)malloc(sizeof(MenuStatusFunc*) * newLength);  
+		if(newFuncs == NULL)
+		{
+			fprintf(stderr, "%s: %d: malloc() fail.\n", __FILE__, __LINE__);
+			abort();
+		}
+		
+		memcpy(newFuncs, menuStatusFuncs, sizeof(MenuStatusFunc*) * menuStatusFuncsLength);
+		free(menuStatusFuncs);
+		menuStatusFuncs = newFuncs;
+		menuStatusFuncsLength = newLength;
+	}
+
+	if (func != NULL)
+	{
+		menuStatusFuncs[win] = func;
+		::glutMenuStatusFunc(menuStatus);
+	}
+	else
+	{
+		menuStatusFuncs[win] = NULL;
+		::glutMenuStatusFunc(NULL);
+	}
+#endif
+}
+
+
+#if (GLUT_API_VERSION >= 3)
+static OverlayDisplayFunc** overlayDisplayFuncs = NULL;
+static int overlayDisplayFuncsLength = 0;
+
+static 
+void 
+overlayDisplay()
+{
+	int win = ::glutGetWindow();
+	OverlayDisplayFunc* overlayDisplayFunc = overlayDisplayFuncs[win];
+	if (overlayDisplayFunc != NULL) overlayDisplayFunc->overlayDisplay();
+}
+#endif
+
+void 
+GLUT::glutOverlayDisplayFunc (::glut4gcj::OverlayDisplayFunc *func)
+{
+	JvInitClass(&GLUT::class$);
+#if (GLUT_API_VERSION >= 3)
+
+	int win = ::glutGetWindow();
+	if (win >= overlayDisplayFuncsLength)
+	{
+		int newLength = overlayDisplayFuncsLength * 2;
+		if (win >= newLength) newLength = win * 2 + 2;
+		OverlayDisplayFunc** newFuncs = (OverlayDisplayFunc**)malloc(sizeof(OverlayDisplayFunc*) * newLength);  
+		if(newFuncs == NULL)
+		{
+			fprintf(stderr, "%s: %d: malloc() fail.\n", __FILE__, __LINE__);
+			abort();
+		}
+		
+		memcpy(newFuncs, overlayDisplayFuncs, sizeof(OverlayDisplayFunc*) * overlayDisplayFuncsLength);
+		free(overlayDisplayFuncs);
+		overlayDisplayFuncs = newFuncs;
+		overlayDisplayFuncsLength = newLength;
+	}
+
+	if (func != NULL)
+	{
+		overlayDisplayFuncs[win] = func;
+		::glutOverlayDisplayFunc(overlayDisplay);
+	}
+#endif
+}
+
+
+
+#if (GLUT_API_VERSION >= 4 || GLUT_XLIB_IMPLEMENTATION >= 9)
+static WindowStatusFunc** windowStatusFuncs = NULL;
+static int windowStatusFuncsLength = 0;
+
+static 
+void 
+windowStatus(int status)
+{
+	int win = ::glutGetWindow();
+	WindowStatusFunc* windowStatusFunc = windowStatusFuncs[win];
+	if (windowStatusFunc != NULL) windowStatusFunc->windowStatus(status);
+}
+#endif
+
+void 
+GLUT::glutWindowStatusFunc (::glut4gcj::WindowStatusFunc *func)
+{
+	JvInitClass(&GLUT::class$);
+#if (GLUT_API_VERSION >= 4 || GLUT_XLIB_IMPLEMENTATION >= 9)
+
+	int win = ::glutGetWindow();
+	if (win >= windowStatusFuncsLength)
+	{
+		int newLength = windowStatusFuncsLength * 2;
+		if (win >= newLength) newLength = win * 2 + 2;
+		WindowStatusFunc** newFuncs = (WindowStatusFunc**)malloc(sizeof(WindowStatusFunc*) * newLength);  
+		if(newFuncs == NULL)
+		{
+			fprintf(stderr, "%s: %d: malloc() fail.\n", __FILE__, __LINE__);
+			abort();
+		}
+		
+		memcpy(newFuncs, windowStatusFuncs, sizeof(WindowStatusFunc*) * windowStatusFuncsLength);
+		free(windowStatusFuncs);
+		windowStatusFuncs = newFuncs;
+		windowStatusFuncsLength = newLength;
+	}
+
+	if (func != NULL)
+	{
+		windowStatusFuncs[win] = func;
+		::glutWindowStatusFunc(windowStatus);
+	}
+#endif
+}
+
+
+#if (GLUT_API_VERSION >= 4 || GLUT_XLIB_IMPLEMENTATION >= 13)
+static KeyboardUpFunc** keyboardUpFuncs = NULL;
+static int keyboardUpFuncsLength = 0;
+
+static 
+void 
+keyboardUp(unsigned char key, int x, int y)
+{
+	int win = ::glutGetWindow();
+	KeyboardUpFunc* keyboardUpFunc = keyboardUpFuncs[win];
+	if (keyboardUpFunc != NULL) keyboardUpFunc->keyboardUp(key, x, y);
+}
+#endif
+
+void 
+GLUT::glutKeyboardUpFunc (::glut4gcj::KeyboardUpFunc *func)
+{
+	JvInitClass(&GLUT::class$);
+
+#if (GLUT_API_VERSION >= 4 || GLUT_XLIB_IMPLEMENTATION >= 13)
+	int win = ::glutGetWindow();
+	if (win >= keyboardUpFuncsLength)
+	{
+		int newLength = keyboardUpFuncsLength * 2;
+		if (win >= newLength) newLength = win * 2 + 2;
+		KeyboardUpFunc** newFuncs = (KeyboardUpFunc**)malloc(sizeof(KeyboardUpFunc*) * newLength);  
+		if(newFuncs == NULL)
+		{
+			fprintf(stderr, "%s: %d: malloc() fail.\n", __FILE__, __LINE__);
+			abort();
+		}
+		
+		memcpy(newFuncs, keyboardUpFuncs, sizeof(KeyboardUpFunc*) * keyboardUpFuncsLength);
+		free(keyboardUpFuncs);
+		keyboardUpFuncs = newFuncs;
+		keyboardUpFuncsLength = newLength;
+	}
+
+	if (func != NULL)
+	{
+		keyboardUpFuncs[win] = func;
+		::glutKeyboardUpFunc(keyboardUp);
+	}
+	else
+	{
+		keyboardUpFuncs[win] = NULL;
+		::glutKeyboardUpFunc(NULL);
+	}
+#endif
+}
+
+#if (GLUT_API_VERSION >= 4 || GLUT_XLIB_IMPLEMENTATION >= 13)
+static SpecialUpFunc** specialUpFuncs = NULL;
+static int specialUpFuncsLength = 0;
+
+static 
+void 
+specialUp(int key, int x, int y)
+{
+	int win = ::glutGetWindow();
+	SpecialUpFunc* specialUpFunc = specialUpFuncs[win];
+	if (specialUpFunc != NULL) specialUpFunc->specialUp(key, x, y);
+}
+#endif
+
+void 
+GLUT::glutSpecialUpFunc (::glut4gcj::SpecialUpFunc *func)
+{
+	JvInitClass(&GLUT::class$);
+#if (GLUT_API_VERSION >= 4 || GLUT_XLIB_IMPLEMENTATION >= 13)
+	int win = ::glutGetWindow();
+	if (win >= specialUpFuncsLength)
+	{
+		int newLength = specialUpFuncsLength * 2;
+		if (win >= newLength) newLength = win * 2 + 2;
+		SpecialUpFunc** newFuncs = (SpecialUpFunc**)malloc(sizeof(SpecialUpFunc*) * newLength);  
+		if(newFuncs == NULL)
+		{
+			fprintf(stderr, "%s: %d: malloc() fail.\n", __FILE__, __LINE__);
+			abort();
+		}
+		
+		memcpy(newFuncs, specialUpFuncs, sizeof(SpecialUpFunc*) * specialUpFuncsLength);
+		free(specialUpFuncs);
+		specialUpFuncs = newFuncs;
+		specialUpFuncsLength = newLength;
+	}
+
+	if (func != NULL)
+	{
+		specialUpFuncs[win] = func;
+		::glutSpecialUpFunc(specialUp);
+	}
+	else
+	{
+		specialUpFuncs[win] = NULL;
+		::glutSpecialUpFunc(NULL);
+	}
+#endif
+}
+
+
+#if (GLUT_API_VERSION >= 4 || GLUT_XLIB_IMPLEMENTATION >= 13)
+static JoystickFunc** joystickFuncs = NULL;
+static int joystickFuncsLength = 0;
+
+static 
+void 
+joystick(unsigned int buttonMask, int x, int y, int z)
+{
+	int win = ::glutGetWindow();
+	JoystickFunc* joystickFunc = joystickFuncs[win];
+	if (joystickFunc != NULL) joystickFunc->joystick(buttonMask, x, y, z);
+}
+#endif
+
+void 
+GLUT::glutJoystickFunc (::glut4gcj::JoystickFunc *func, jint pollInterval)
+{
+	JvInitClass(&GLUT::class$);
+#if (GLUT_API_VERSION >= 4 || GLUT_XLIB_IMPLEMENTATION >= 13)
+	int win = ::glutGetWindow();
+	if (win >= joystickFuncsLength)
+	{
+		int newLength = joystickFuncsLength * 2;
+		if (win >= newLength) newLength = win * 2 + 2;
+		JoystickFunc** newFuncs = (JoystickFunc**)malloc(sizeof(JoystickFunc*) * newLength);  
+		if(newFuncs == NULL)
+		{
+			fprintf(stderr, "%s: %d: malloc() fail.\n", __FILE__, __LINE__);
+			abort();
+		}
+		
+		memcpy(newFuncs, joystickFuncs, sizeof(JoystickFunc*) * joystickFuncsLength);
+		free(joystickFuncs);
+		joystickFuncs = newFuncs;
+		joystickFuncsLength = newLength;
+	}
+
+	if (func != NULL)
+	{
+		joystickFuncs[win] = func;
+		::glutJoystickFunc(joystick, pollInterval);
+	}
+	else
+	{
+		joystickFuncs[win] = NULL;
+		::glutJoystickFunc(NULL, pollInterval);
+	}
+#endif
+}
+
 void
 GLUT::glutSetColor (jint cell, jfloat red, jfloat green, jfloat blue)
 {
@@ -1301,6 +1818,77 @@ GLUT::glutLayerGet (jint info)
 	return result;
 }
 
+void
+GLUT::glutBitmapCharacter (BitmapFont* font, jchar character)
+{
+	JvInitClass(&GLUT::class$);
+	void* nativeFont = (void*)font->implementation;
+	::glutBitmapCharacter(nativeFont, (int)character);
+}
+
+jint
+GLUT::glutBitmapWidth (BitmapFont* font, jchar character)
+{
+	JvInitClass(&GLUT::class$);
+	void* nativeFont = (void*)font->implementation;
+	return ::glutBitmapWidth(nativeFont, (int)character);
+}
+
+
+void
+GLUT::glutStrokeCharacter (StrokeFont* font, jchar character)
+{
+	JvInitClass(&GLUT::class$);
+	void* nativeFont = (void*)font->implementation;
+	::glutStrokeCharacter(nativeFont, (int)character);
+}
+
+jint
+GLUT::glutStrokeWidth (StrokeFont* font, jchar character)
+{
+	JvInitClass(&GLUT::class$);
+	void* nativeFont = (void*)font->implementation;
+	return ::glutStrokeWidth(nativeFont, (int)character);
+}
+
+jint
+GLUT::glutBitmapLength (BitmapFont* font, String* string)
+{
+	JvInitClass(&GLUT::class$);
+	jint width = -1;
+#if (GLUT_API_VERSION >= 4 || GLUT_XLIB_IMPLEMENTATION >= 9)
+	void* nativeFont = (void*)font->implementation;
+
+	jbyteArray byteArray = string->getBytes();
+	jbyte* bytes = elements(byteArray);
+	unsigned char buffer[byteArray->length+1];
+	memcpy(buffer, bytes, byteArray->length);
+	buffer[byteArray->length] = '\0';
+
+	width = ::glutBitmapLength(nativeFont, buffer);
+#endif
+	return width;
+}
+
+jint
+GLUT::glutStrokeLength (StrokeFont* font, String* string)
+{
+	JvInitClass(&GLUT::class$);
+	jint width = -1;
+#if (GLUT_API_VERSION >= 4 || GLUT_XLIB_IMPLEMENTATION >= 9)
+	void* nativeFont = (void*)font->implementation;
+
+	jbyteArray byteArray = string->getBytes();
+	jbyte* bytes = elements(byteArray);
+	unsigned char buffer[byteArray->length+1];
+	memcpy(buffer, bytes, byteArray->length);
+	buffer[byteArray->length] = '\0';
+
+	width = ::glutStrokeLength(nativeFont, buffer);
+#endif
+	return width;
+}
+
 void
 GLUT::glutWireSphere (jdouble radius, jint slices, jint stacks)
 {
