@@ -24,6 +24,7 @@ public class StrokeString extends GLUT
 	private void drawText()
 	{
 		glPushAttrib(GL_ENABLE_BIT|GL_LINE_BIT);
+		glColor3d(0.0, 1.0, 1.0);
 
 		glEnable(GL_BLEND);
 		glEnable(GL_LINE_SMOOTH);
@@ -33,8 +34,6 @@ public class StrokeString extends GLUT
 		glPushMatrix();
 		glLoadIdentity();
 		glTranslated(0.0, 0.0, 0.0);
-//		System.out.println("w = " + glutStrokeWidth(GLUT_STROKE_ROMAN, "Hello"));
-//		System.out.println("w = " + glutStrokeLength(GLUT_STROKE_ROMAN, "Hello"));
 		glutStrokeCharacter(GLUT_STROKE_ROMAN, "Hello");
 		glPopMatrix();
 		glPopAttrib();
